@@ -9,9 +9,15 @@ our @ISA = qw(Exporter);
 
 our @EXPORT_OK = qw(c2xs);
 
-our $VERSION = 0.17;
+our $VERSION = 0.18;
 
-my $config_options;       
+my $config_options;
+
+our @allowable_config_keys = ('AUTOWRAP', 'AUTO_INCLUDE', 'CODE', 'TYPEMAPS', 'LIBS', 'INC', 
+        'WRITE_MAKEFILE_PL', 'BUILD_NOISY', 'BOOT', 'BOOT_F', 'EXPORT_ALL', 'EXPORT_OK_ALL',
+        'EXPORT_TAGS_ALL', 'MAKE', 'PREFIX', 'PREREQ_PM', 'CCFLAGS', 'CCFLAGSEX', 'LD', 'LDDLFLAGS',
+        'MYEXTLIB', 'OPTIMIZE', 'PRE_HEAD', 'CC', 'SRC_LOCATION', '_TESTING', 'USE', 'USING',
+        'WRITE_PM', 'VERSION');       
 
 sub c2xs {
     eval {require "Inline/C.pm"};
@@ -48,6 +54,10 @@ sub c2xs {
       die "You can provide either CODE *or* SRC_LOCATION arguments ... but not *both*";
     }
 
+    if(exists($config_options->{BOOT}) && exists($config_options->{BOOT_F})) {
+      die "You can provide either BOOT *or* BOOT_F arguments ... but not *both*";
+    }
+
     if(exists($config_options->{CODE})) {
       $code = $config_options->{CODE};
       if($code =~ /inline_stack_vars/i) {$need_inline_h = 1 }
@@ -125,6 +135,17 @@ sub c2xs {
 
     if($config_options->{BOOT}) {$o->{ILSM}{XS}{BOOT} = $config_options->{BOOT}} 
 
+    if($config_options->{BOOT_F}) {
+      my $code;
+      open(RD, "<", $config_options->{BOOT_F}) or die "Can't open ", $config_options->{BOOT_F}, " for reading: $!";
+      while(<RD>) { 
+           $code .= $_;
+           if($_ =~ /inline_stack_vars/i) {$need_inline_h = 1}
+      }
+      close(RD) or die "Can't close ", $config_options->{BOOT_F}, " after reading: $!";
+      $o->{ILSM}{XS}{BOOT} = $code;
+    }
+
     # This is what Inline::C does with the MAKE parameter ... so we'll do the same.
     # Not sure that this achieves anything in the context of InlineX::C2XS.
     if($config_options->{MAKE}) {$o->{ILSM}{MAKE} = $config_options->{MAKE}}
@@ -161,12 +182,32 @@ sub c2xs {
 
     Inline::C::validate($o);
 
+    if($config_options->{PRE_HEAD}) {
+      my $v = $config_options->{PRE_HEAD};
+      #{ # open scope
+      #  no warnings 'newline';
+        unless( -f $v) {
+          $o->{ILSM}{AUTO_INCLUDE} = $v . "\n" . $o->{ILSM}{AUTO_INCLUDE};
+        }
+        else {
+          my $insert;
+          open RD, '<', $v or die "Couldn't open $v for reading: $!";
+          while(<RD>) {$insert .= $_}
+          close RD or die "Couldn't close $v after reading: $!";
+          $o->{ILSM}{AUTO_INCLUDE} = $insert . "\n" . $o->{ILSM}{AUTO_INCLUDE};
+        }
+      #} # close scope
+    }
+
     if($config_options->{AUTO_INCLUDE}) {$o->{ILSM}{AUTO_INCLUDE} .= $config_options->{AUTO_INCLUDE} . "\n"}
 
     if($config_options->{CC}) {$o->{ILSM}{MAKEFILE}{CC} = $config_options->{CC}}
 
     if($config_options->{CCFLAGS}) {$o->{ILSM}{MAKEFILE}{CCFLAGS} = " " . $config_options->{CCFLAGS}}
 
+    if($config_options->{CCFLAGSEX}) {$o->{ILSM}{MAKEFILE}{CCFLAGS} = $Config{ccflags} . " "
+                                                                      . $config_options->{CCFLAGSEX}}
+
     if(exists($config_options->{INC})) {
       if(ref($config_options->{INC}) eq 'ARRAY') {$o->{ILSM}{MAKEFILE}{INC} = join ' ', @{$config_options->{INC}};}
       else {$o->{ILSM}{MAKEFILE}{INC} = $config_options->{INC};}
@@ -240,12 +281,9 @@ sub _build {
 }
 
 sub _check_config_keys {
-    for('AUTOWRAP', 'AUTO_INCLUDE', 'CODE', 'TYPEMAPS', 'LIBS', 'INC', 
-        'WRITE_MAKEFILE_PL', 'BUILD_NOISY', 'BOOT', 'EXPORT_ALL', 'EXPORT_OK_ALL',
-        'EXPORT_TAGS_ALL', 'MAKE', 'PREFIX', 'PREREQ_PM', 'CCFLAGS', 'LD', 'LDDLFLAGS',
-        'MYEXTLIB', 'OPTIMIZE', 'CC', 'SRC_LOCATION', '_TESTING', 'USE', 'USING',
-        'WRITE_PM', 'VERSION',)
-       {return 1 if $_ eq $_[0]} # it's a valid config option
+    for(@allowable_config_keys) {
+        return 1 if $_ eq $_[0]; # it's a valid config option
+    }
     return 0;                    # it's an invalid config option
 }
 
@@ -481,6 +519,14 @@ InlineX::C2XS - Convert from Inline C code to XS.
     BOOT => 'printf("Hello .. from bootstrap\n");',
   ----
 
+ BOOT_F
+   Specifies a file containing C code to be executed in the XS BOOT
+   section. 
+   eg:
+
+    BOOT_F => '/home/me/boot_code.ext',
+  ----
+
   BUILD_NOISY
    Is set to a true value, by default. Setting to a false value will
    mean that progress messages generated by Inline::C are suppressed. eg:
@@ -500,7 +546,15 @@ InlineX::C2XS - Convert from Inline C code to XS.
    you'll probably want to re-specify it.) It makes sense to assign this
    key only when WRITE_MAKEFILE_PL is set to a true value. eg:
 
-    CCFLAGS => '-DMY_DEFINE ' . $Config{ccflags},
+    CCFLAGS => $Config{ccflags} . ' -DMY_DEFINE',
+  ----
+
+  CCFLAGSEX
+   Add compiler flags to existing flags.
+   It makes sense to assign this key only when WRITE_MAKEFILE_PL is set to
+   a true value. eg:
+
+    CCFLAGSEX => '-DMY_DEFINE ',
   ----
 
   CODE
@@ -597,6 +651,17 @@ InlineX::C2XS - Convert from Inline C code to XS.
     PREFIX => 'FOO_',
   ----
 
+  PRE_HEAD
+
+   Specifies code that will precede the inclusion of all files specified
+   in AUTO_INCLUDE (ie EXTERN.h, perl.h, XSUB.h, INLINE.h and anything
+   else that might have been added to AUTO_INCLUDE by the user). If the
+   specified value identifies a file, the contents of that file will be
+   inserted, otherwise the specified value is inserted.
+
+    PRE_HEAD => $code_or_filename;
+  ----
+
   PREREQ_PM
    Makes sense to specify this only if WRITE_MAKEFILE_PL is set to true.
    The string to which PREREQ_PM is set will be reproduced as is in the
@@ -683,7 +748,7 @@ InlineX::C2XS - Convert from Inline C code to XS.
 
   This program is free software; you may redistribute it and/or 
   modify it under the same terms as Perl itself.
-  Copyright 2006-2009, 2010-11,Sisyphus
+  Copyright 2006-2009, 2010-12,Sisyphus
 
 
 =head1 AUTHOR
